import { useRequest } from "ahooks";
import React, { useState } from "react";
import styles from "./index.less";
import ReactECharts from "echarts-for-react";
import { createFromIconfontCN } from "@ant-design/icons";
import * as echarts from "echarts";
import { Button } from "antd";
import "echarts/lib/component/graphic";
import { ScrollBoard } from "@jiaminghi/data-view-react";
import { doFetch } from "@/utils/doFetch";
import Threedire from "./threedire";
import dayjs from "dayjs";

const IconFont = createFromIconfontCN({
    scriptUrl: "//at.alicdn.com/t/c/font_3649650_01jk3haanj83.js",
  }),
  statuscolor = {
    紧急: "#F31A1A",
    重要: "#F6BD16",
    普通: "#C7C8CA",
  };

const Item = ({ title, children }) => {
  return (
    <div className={styles.item}>
      <p className={styles?.title}>
        <IconFont
          type="icon-lingxing"
          style={{ fontSize: 12, marginRight: 10 }}
        />
        {title}
      </p>

      {children}
    </div>
  );
};

export default function HomePage() {
  const leftone = useRequest(
    async () => {
      //format result
      let res = await doFetch({
          url: "/ngic-statistics/deviceStatus/queryEquipmentStatusChart",
          params: {},
        }),
        result = res?.data?.dataList ?? [];
      return result;
    },
    {
      pollingInterval: 20000,
    }
  );
  const lefttwo = useRequest(
    async () => {
      //format result
      let res = await doFetch({
          url: "/ngic-statistics/producePoolDay/queryShopBoard",
          params: {},
        }),
        result = res?.data?.dataList ?? [];
      return result;
    },
    {
      pollingInterval: 20000,
    }
  );
  const leftthree = useRequest(
    async () => {
      //format result
      let res = await doFetch({
          url: "/ngic-statistics/deviceStatus/shopUsedRate",
          params: {},
        }),
        result = res?.data?.dataList ?? [];
      return result;
    },
    {
      pollingInterval: 20000,
    }
  );

  const rightone = useRequest(
    async () => {
      //format result
      let res = await doFetch({
          url: "/ngic-statistics/alarmPoolDay/queryAlarmList",
          params: {},
        }),
        result = res?.data?.dataList ?? [];
      return result;
    },
    {
      pollingInterval: 20000,
    }
  );

  const righttwo = useRequest(
    async () => {
      //format result
      let res = await doFetch({
          url: "/ngic-statistics/electricityPoolDay/queryShopBoard",
          params: {},
        }),
        result = res?.data?.dataList ?? [];
      return result;
    },
    {
      pollingInterval: 20000,
    }
  );

  const rightthree = useRequest(
    async () => {
      //format result
      let res = await doFetch({
          url: "/ngic-statistics/alarmPoolDay/queryShopBoard",
          params: {},
        }),
        result = res?.data?.dataList ?? [];
      return result;
    },
    {
      pollingInterval: 20000,
    }
  );

  return (
    <div className={styles?.container}>
      <div className={styles?.left}>
        <Item title={"设备状态"}>
          <ReactECharts
            style={{ width: "100%", height: "100%", marginTop: 0 }}
            option={{
              legend: {
                orient: "vertical", //垂直显示
                y: "center", //延Y轴居中
                x: "right", //居右显示
                padding: 40, //调节legend的位置
                textStyle: {
                  fontSize: 14, //字体大小
                  color: "#ffffff", //字体颜色
                },
                icon: "circle",
                formatter: function (name) {
                  //图例后添加数值
                  let data = leftone.data ?? [];
                  let tarValue;
                  for (let i = 0; i < data.length; i++) {
                    if (data[i].name === name) {
                      tarValue = data[i].value;
                    }
                  }
                  return name + ": " + tarValue;
                },
              },
              tooltip: {
                trigger: "item",
              },
              color: ["#6AE49B", "#4887E0", "#8D8D8D", "#F31A1A"],
              series: [
                {
                  name: "设备状态",
                  type: "pie",
                  radius: [40, 80],
                  center: ["34%", "50%"],
                  label: {
                    show: false,
                    position: "center",
                  },
                  emphasis: {
                    label: {
                      show: true,
                      fontSize: "28",
                      fontWeight: "bold",
                    },
                  },
                  itemStyle: {
                    borderRadius: 8,
                  },
                  data: leftone.data,
                },
              ],
            }}
            notMerge={true}
            lazyUpdate={true}
            theme={"theme_name"}
          />
        </Item>
        <Item title={"生产统计"}>
          <ReactECharts
            style={{ width: "100%", height: "100%", marginTop: -28 }}
            option={{
              xAxis: {
                data: lefttwo?.data?.map((it) => it.key),
                splitLine: {
                  show: false,
                },
                axisLabel: {
                  color: "#bbbbbb",
                },
                axisTick: {
                  show: false,
                },
                axisLine: {
                  show: false,
                },
                z: 10,
              },
              yAxis: {
                axisLine: {
                  show: false,
                },
                axisTick: {
                  show: false,
                },
                axisLabel: {
                  color: "#bbbbbb",
                  show: true,
                  formatter: "{value}件",
                },
                splitLine: {
                  lineStyle: {
                    color: ["rgba(255,255,255,0.2)"],
                    type: "solid",
                  },
                },
              },
              dataZoom: [
                {
                  type: "inside",
                },
              ],
              tooltip: {
                trigger: "item",
              },
              grid: {
                top: "58px",
                left: "12px",
                right: "5%",
                bottom: "28px",
                containLabel: true,
              },
              series: [
                {
                  type: "bar",
                  itemStyle: {
                    color: new echarts.graphic.LinearGradient(0, 0, 0, 1, [
                      { offset: 0, color: "rgba(106,228,155,0.4)" },
                      { offset: 0.2, color: "rgba(106,228,155,0.2)" },
                      { offset: 1, color: "rgba(106,228,155,0)" },
                    ]),
                    borderWidth: 1,
                    borderColor: "#64DB9B",
                  },
                  // emphasis: {
                  //   itemStyle: {
                  //     color: new echarts.graphic.LinearGradient(0, 0, 0, 1, [
                  //       { offset: 0, color: "#2378f7" },
                  //       { offset: 0.7, color: "#2378f7" },
                  //       { offset: 1, color: "#83bff6" },
                  //     ]),
                  //   },
                  // },
                  data: lefttwo?.data?.map((it) => it.value),
                },
              ],
            }}
            notMerge={true}
            lazyUpdate={true}
            theme={"theme_name"}
          />
        </Item>
        <Item title={"车间利用率"}>
          <ReactECharts
            style={{ width: "100%", height: "100%", marginTop: -28 }}
            option={{
              tooltip: {
                trigger: "item",
              },
              xAxis: {
                axisLabel: {
                  color: "#bbbbbb",
                },
                type: "category",
                boundaryGap: false,
                data: leftthree?.data?.map?.((it) => it.name),
              },
              grid: {
                top: "58px",
                left: "12px",
                right: "5%",
                bottom: "28px",
                containLabel: true,
              },
              yAxis: {
                type: "value",
                axisLabel: {
                  color: "#bbbbbb",
                  show: true,
                  formatter: "{value}%",
                },
                splitLine: {
                  lineStyle: {
                    color: ["rgba(255,255,255,0.2)"],
                    type: "solid",
                  },
                },
              },
              series: [
                {
                  data: leftthree?.data?.map?.((it) => it.value),
                  type: "line",
                  lineStyle: {
                    color: "rgba(72, 135, 244,0.4)",
                    width: 1,
                    type: "dashed",
                  },
                  areaStyle: {
                    color: new echarts.graphic.LinearGradient(0, 0, 0, 1, [
                      {
                        offset: 0,
                        color: "rgba(72, 135, 244,0.4)",
                      },
                      {
                        offset: 1,
                        color: "rgba(72, 135, 244,0)",
                      },
                    ]),
                  },
                },
              ],
            }}
            notMerge={true}
            lazyUpdate={true}
            theme={"theme_name"}
          />
        </Item>
      </div>
      <div className={styles?.bac}>
        <Threedire></Threedire>
      </div>

      <div className={styles?.right}>
        <Item title={"告警信息"}>
          <div style={{ padding: "0 18px" }}>
            <ScrollBoard
              config={{
                headerBGC: "tranparent",
                oddRowBGC: "tranparent",
                evenRowBGC: "tranparent",
                header: ["告警时间", "严重", "设备名称", "告警"],
                data: rightone?.data?.map((it) => {
                  return [
                    dayjs(it?.alarmTime).format("MM-DD HH:mm"),
                    `<span style="color:${statuscolor[it?.severityName]}">${
                      it?.severityName
                    }</span>`,
                    `<a title="${it?.deviceName}" style="color:rgba(255,255,255,0.5)">${it?.deviceName}</a>`,
                    `<a title="${it?.details}" style="color:rgba(255,255,255,0.5)">${it?.details}</a>`,
                  ];
                }),
                rowNum: 6,
                columnWidth: [104, 50],
              }} //重要状态字体颜色：#F6BD16、紧急状态字体颜色：#F31A1A；普通状态字体颜色：#C7C8CA。
              style={{ width: "100%", height: "100%" }}
            />
          </div>
        </Item>
        <Item title={"用电量"}>
          <ReactECharts
            style={{ width: "100%", height: "100%", marginTop: -28 }}
            option={{
              xAxis: {
                data: righttwo?.data?.map((it) => it.key),
                splitLine: {
                  show: false,
                },
                axisLabel: {
                  color: "#bbbbbb",
                },
                axisTick: {
                  show: false,
                },
                axisLine: {
                  show: false,
                },
                z: 10,
              },
              yAxis: {
                axisLine: {
                  show: false,
                },
                axisTick: {
                  show: false,
                },
                axisLabel: {
                  color: "#bbbbbb",
                  show: true,
                  formatter: "{value}度",
                },
                splitLine: {
                  lineStyle: {
                    color: ["rgba(255,255,255,0.2)"],
                    type: "solid",
                  },
                },
              },
              dataZoom: [
                {
                  type: "inside",
                },
              ],
              tooltip: {
                trigger: "item",
              },
              grid: {
                top: "58px",
                left: "12px",
                right: "5%",
                bottom: "28px",
                containLabel: true,
              },
              series: [
                {
                  type: "bar",
                  itemStyle: {
                    color: new echarts.graphic.LinearGradient(0, 0, 0, 1, [
                      { offset: 0, color: "rgba(72,135,244,0.8)" },
                      { offset: 0.2, color: "rgba(72,135,244,0.4)" },
                      { offset: 1, color: "rgba(72,135,244,0)" },
                    ]),
                  },
                  // emphasis: {
                  //   itemStyle: {
                  //     color: new echarts.graphic.LinearGradient(0, 0, 0, 1, [
                  //       { offset: 0, color: "#2378f7" },
                  //       { offset: 0.7, color: "#2378f7" },
                  //       { offset: 1, color: "#83bff6" },
                  //     ]),
                  //   },
                  // },
                  data: righttwo?.data?.map((it) => it.value),
                },
              ],
            }}
            notMerge={true}
            lazyUpdate={true}
            theme={"theme_name"}
          />
        </Item>
        <Item title={"故障统计"}>
          <ReactECharts
            style={{ width: "100%", height: "100%", marginTop: -28 }}
            option={{
              tooltip: {
                trigger: "item",
              },
              xAxis: {
                axisLabel: {
                  color: "#bbbbbb",
                },
                type: "category",
                boundaryGap: false,
                data: rightthree?.data?.map?.((it) => it.key),
              },
              grid: {
                top: "58px",
                left: "12px",
                right: "5%",
                bottom: "28px",
                containLabel: true,
              },
              yAxis: {
                type: "value",
                axisLabel: {
                  color: "#bbbbbb",
                  show: true,
                  formatter: "{value}次",
                },
                splitLine: {
                  lineStyle: {
                    color: ["rgba(255,255,255,0.2)"],
                    type: "solid",
                  },
                },
              },
              series: [
                {
                  data: rightthree?.data?.map?.((it) => it.value),
                  type: "line",
                  lineStyle: {
                    color: "rgba(243, 26, 25,0.4)",
                    width: 1,
                    type: "dashed",
                  },
                  itemStyle: {
                    borderColor: "rgba(243, 26, 25)",
                    color: "rgba(243, 26, 25)",
                  },
                  areaStyle: {
                    color: new echarts.graphic.LinearGradient(0, 0, 0, 1, [
                      {
                        offset: 0,
                        color: "rgba(243, 26, 25,0.4)",
                      },
                      {
                        offset: 1,
                        color: "rgba(243, 26, 25,0)",
                      },
                    ]),
                  },
                },
              ],
            }}
            notMerge={true}
            lazyUpdate={true}
            theme={"theme_name"}
          />
        </Item>
      </div>
    </div>
  );
}
